@@ -80,6 +80,23 @@ impl Migrate for AnyConnection {
         }
     }
 
+    #[allow(deprecated)]
+    fn version(&mut self) -> BoxFuture<'_, Result<Option<(i64, bool)>, MigrateError>> {
+        match &mut self.0 {
+            #[cfg(feature = "postgres")]
+            AnyConnectionKind::Postgres(conn) => conn.version(),
+
+            #[cfg(feature = "sqlite")]
+            AnyConnectionKind::Sqlite(conn) => conn.version(),
+
+            #[cfg(feature = "mysql")]
+            AnyConnectionKind::MySql(conn) => conn.version(),
+
+            #[cfg(feature = "mssql")]
+            AnyConnectionKind::Mssql(_conn) => unimplemented!(),
+        }
+    }
+
     fn dirty_version(&mut self) -> BoxFuture<'_, Result<Option<i64>, MigrateError>> {
         match &mut self.0 {
             #[cfg(feature = "postgres")]
@@ -96,6 +113,29 @@ impl Migrate for AnyConnection {
         }
     }
 
+    #[allow(deprecated)]
+    fn validate<'e: 'm, 'm>(
+        &'e mut self,
+        migration: &'m Migration,
+    ) -> BoxFuture<'m, Result<(), MigrateError>> {
+        match &mut self.0 {
+            #[cfg(feature = "postgres")]
+            AnyConnectionKind::Postgres(conn) => conn.validate(migration),
+
+            #[cfg(feature = "sqlite")]
+            AnyConnectionKind::Sqlite(conn) => conn.validate(migration),
+
+            #[cfg(feature = "mysql")]
+            AnyConnectionKind::MySql(conn) => conn.validate(migration),
+
+            #[cfg(feature = "mssql")]
+            AnyConnectionKind::Mssql(_conn) => {
+                let _ = migration;
+                unimplemented!()
+            }
+        }
+    }
+
     fn list_applied_migrations(
         &mut self,
     ) -> BoxFuture<'_, Result<Vec<AppliedMigration>, MigrateError>> {
