@@ -13,9 +13,9 @@ mod arguments;
 pub(crate) mod column;
 mod connection;
 mod database;
-mod done;
 mod kind;
 mod options;
+mod query_result;
 pub(crate) mod row;
 mod statement;
 mod transaction;
@@ -31,11 +31,10 @@ pub use column::{AnyColumn, AnyColumnIndex};
 pub use connection::AnyConnection;
 pub use database::Any;
 pub use decode::AnyDecode;
-pub use done::AnyDone;
 pub use encode::AnyEncode;
 pub use kind::AnyKind;
 pub use options::AnyConnectOptions;
-pub use r#type::AnyType;
+pub use query_result::AnyQueryResult;
 pub use row::AnyRow;
 pub use statement::AnyStatement;
 pub use transaction::AnyTransactionManager;
@@ -54,7 +53,6 @@ impl_acquire!(Any, AnyConnection);
 impl_column_index_for_row!(AnyRow);
 impl_column_index_for_statement!(AnyStatement);
 impl_into_maybe_pool!(Any, AnyConnection);
-impl_map_row!(Any, AnyRow);
 
 // required because some databases have a different handling of NULL
 impl_encode_for_option!(Any);
